/*
Beregost Crash Fixer
Copyright (C) 2008-2010  Raphael Wong

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package asc64.beregost;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.math.BigInteger;

public class BeregostUtils {

  public static final String SAV_FILE = "sav";

  public static String getExtension(File file) {
    String s = file.getName();
    int i = s.lastIndexOf('.');
    if (i > 0 && i < s.length() - 1) {
      return s.substring(i+1).toLowerCase();
    }
    return "null";
  }

  //reads a long value at offset, converts endian-ness and returns the value
  public static int readLong(RandomAccessFile file, long offset) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[8];
    file.read(buffer);
    byte[] bufferReverse = {buffer[7], buffer[6], buffer[5], buffer[4], buffer[3], buffer[2], buffer[1], buffer[0]};
    return new BigInteger(bufferReverse).intValue();
  }

  //writes a long value in little-endian, then readLong() converts endian-ness, before the value is written again
  public static void writeLong(RandomAccessFile file, long offset, long value) throws IOException {
    file.seek(offset);
    file.writeLong(value);
    value = readLong(file, offset);
    file.seek(offset);
    file.writeLong(value);
  }

  //reads an int value at offset, converts endian-ness and returns the value
  public static int readInt(RandomAccessFile file, long offset) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[4];
    file.read(buffer);
    byte[] bufferReverse = {buffer[3], buffer[2], buffer[1], buffer[0]};
    return new BigInteger(bufferReverse).intValue();
  }

  //writes an int value in little-endian, then readShort() converts endian-ness, before the value is written again
  public static void writeInt(RandomAccessFile file, long offset, int value) throws IOException {
    file.seek(offset);
    file.writeInt(value);
    value = readInt(file, offset);
    file.seek(offset);
    file.writeInt(value);
  }

  //reads a short value at offset, converts endian-ness and returns the value
  public static short readShort(RandomAccessFile file, long offset) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[2];
    file.read(buffer);
    byte[] bufferReverse = {buffer[1], buffer[0]};
    return new BigInteger(bufferReverse).shortValue();
  }

  //writes a short value in little-endian, then readShort() converts endian-ness, before the value is written again
  //value parameter is an int rather than a short because RandomAccessFile.writeShort requires an int argument
  public static void writeShort(RandomAccessFile file, long offset, int value) throws IOException {
    file.seek(offset);
    file.writeShort(value);
    value = readShort(file, offset);
    file.seek(offset);
    file.writeShort(value);
  }

  //returns a byte value read at offset
  public static byte readByte(RandomAccessFile file, long offset) throws IOException {
    file.seek(offset);
    return file.readByte();
  }

  //returns a byte array of size length starting reading at offset
  public static byte[] readByte(RandomAccessFile file, long offset, int length) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[length];
    file.read(buffer);
    return buffer;
  }

  //returns an 8-character read at offset as a String
  public static String readAscii(RandomAccessFile file, long offset) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[8];
    file.read(buffer);
    char[] bufferChar = new char[buffer.length];
    for (int i = 0; i < buffer.length; i++) {
      bufferChar[i] = (char)buffer[i];
    }
    return String.valueOf(bufferChar);
  }

  //returns a custom length character read at offset as a String
  public static String readAscii(RandomAccessFile file, long offset, int length) throws IOException {
    file.seek(offset);
    byte[] buffer = new byte[length];
    file.read(buffer);
    char[] bufferChar = new char[buffer.length];
    for (int i = 0; i < buffer.length; i++) {
      bufferChar[i] = (char)buffer[i];
    }
    return String.valueOf(bufferChar);
  }

}
